/*
 =================================
  Sequencer med variabelt delay
  ================================
  Forfatter: Lars Petersen OZ1BXM
  E-mail: oz1bxm@pobox.com
  =================================
  Copyright: Denne sketch er i 
  public domain og må gerne anvendes 
  til private formål.
  ==================================
  Instruktion:
  Delay sættes med jumpers på D2-D5 og
  værdien ganges med 20. Jumperne
  kan sættes til 0-15 (0 er ugyldig)
  Delay vil ligge i området fra 20 
  til 300 millisekunder i spring på 20.
  
  Eksempel: 
  Jumpers ved D02-D05 sættes til 8
  Delay bliver således 8 * 20 = 160 millisekunder
  
  Værdien af delayValue beregnes når programmet starter.
  delayValue sendes 1 gang via Serial Monitor, som sættes til
  "Both NL & CR" og "9600 baud" i Arduino IDE.
  
  Du kan tjekke delayValue ved at trykke 
  på Arduino Nanos "Reset" knap, så udlæses 
  delayValue via Serial Monitor.
  */
  
  // Define global variables: 
  boolean pinValue;
  boolean oldPTT;  // LOW is TX; HIGH is RX
  boolean newPTT;  // same
  boolean new1PTT; // temporary value
  int fourBitValue;
  int delayValue;
  
  // Define global constants: 
  const int factor = 20; //multiply by 20
  const int pinD2 = 2; //pin D02, use internal pull-up
  const int pinD3 = 3; //pin D03, use internal pull-up
  const int pinD4 = 4; //pin D04, use internal pull-up
  const int pinD5 = 5; //pin D05, use internal pull-up 
  const int pinPTT = 12; //pin D12 
  const int LED1 = 6; //pin D06
  const int LED2 = 7; //pin D07
  const int LED3 = 8; //pin D08
  const int event1 = 9; //pin D09
  const int event2 = 10; //pin D10
  const int event3 = 11; //pin D11 
  const int debounceDelay = 20; //20 milliseconds
  
void setup() {
  // Configure 4 pins (used for jumpers)  
  pinMode(pinD2, INPUT_PULLUP);
  digitalWrite(pinD2, HIGH);    //turn on pull-up resistor
  pinMode(pinD3, INPUT_PULLUP);
  digitalWrite(pinD3, HIGH);    //turn on pull-up resistor
  pinMode(pinD4, INPUT_PULLUP);
  digitalWrite(pinD4, HIGH);    //turn on pull-up resistor
  pinMode(pinD5, INPUT_PULLUP);
  digitalWrite(pinD5, HIGH);    //turn on pull-up resistor

  // Configure PTT pin  
  pinMode(pinPTT, INPUT);       // pinPTT is input 
   
   // Configure built-in LED  
  pinMode(LED_BUILTIN, OUTPUT); // D13 LED monitors PTT

  // Configure LED pins  
  pinMode(LED1, OUTPUT); //LED1 pin
  pinMode(LED2, OUTPUT); //LED2 pin
  pinMode(LED3, OUTPUT); //LED3 pin
  
  // Configure digital pins   
  pinMode(event1, OUTPUT); // Event1 pin
  pinMode(event2, OUTPUT); // Event2 pin
  pinMode(event3, OUTPUT); // Event3 pin
  
  // Set RX mode
  oldPTT = HIGH; // set RX mode 
  sequenceRX();  // set relevant ports to RX mode

  // Calculate delay
  calculateDelayValue(); //calculate delayValue 
  
  // Send delayValue to PC via serial interface
  Serial.begin(9600); //speed 9600 baud  
  Serial.print("Delay = "); // debug   
  Serial.print(delayValue); // debug   
  Serial.println(" millisekunder"); // debug   
}

void loop() {   
  // decide what to do if PTT status has changed
  newPTT = digitalRead(pinPTT);  // read PTT pin
  if (newPTT != oldPTT) {     // PTT has changed, but not debounced yet 
    delay(debounceDelay);     // wait and see if pinPTT is still the same
    new1PTT = digitalRead(pinPTT); // read PTT pin again
    if (new1PTT == newPTT) {  // debouncing done, new state is stable         
      if (newPTT) {           // newPTT HIGH => change to RX mode
        digitalWrite(LED_BUILTIN, LOW);  // built-in LED turned OFF
        sequenceRX();         // see details in the function below      
        oldPTT = newPTT;      // save the new PTT status
      }
      else {                  // newPTT LOW => change to TX mode
        digitalWrite(LED_BUILTIN, HIGH); // built-in LED turned ON
        sequenceTX();         // see details in the function below
        oldPTT = newPTT;      // save the new PTT status 
      }  
    }  
  } 
}

//////////////////////////////////////////////////////////////////

void calculateDelayValue() {   // read D02-D05 and calculate delayValue
  fourBitValue = 0;            // initialize fourBitValue
  // read D02-D05 and copy each bit into four_bit_value 
  pinValue = digitalRead(pinD5);       // read bit 0 at D05
  bitWrite(fourBitValue,0,pinValue);   // write the bit into fourBitValue
  pinValue = digitalRead(pinD4);       // read bit 1 at D04
  bitWrite(fourBitValue,1,pinValue);   // write the bit into fourBitValue
  pinValue = digitalRead(pinD3);       // read bit 2 at D03
  bitWrite(fourBitValue,2,pinValue);   // write the bit into fourBitValue
  pinValue = digitalRead(pinD2);       // read bit 3 at D02
  bitWrite(fourBitValue,3,pinValue);   // write the bit into fourBitValue
  // secure that delay=0 is replaced with delay=10  
  if (fourBitValue == 0) { // if fourBitValue is 0 (illegal value) 
    fourBitValue = 8;  //set fourBitValue = 8 (delay will then be 160 ms) 
    }
  // calculate delayValue 
  delayValue = fourBitValue * factor;  // calculate delayValue
}

void sequenceTX() { // activate Event1 => Event2 => Event3
  digitalWrite(LED1, HIGH);    // LED1 ON
  digitalWrite(event1, LOW);   // event1 LOW for TX
  delay(delayValue);
  digitalWrite(LED2, HIGH);    // LED2 ON
  digitalWrite(event2, HIGH);  // event2 HIGH for TX
  delay(delayValue);
  digitalWrite(LED3, HIGH);    // LED3 ON
  digitalWrite(event3, HIGH);  // event3 HIGH for TX
}

void sequenceRX() { // deactivate Event3 => Event2 => Event1
  digitalWrite(LED3, LOW);     // LED3 OFF
  digitalWrite(event3, LOW);   // event3 LOW for RX
  delay(delayValue);
  digitalWrite(LED2, LOW);     // LED2 OFF
  digitalWrite(event2, LOW);   // event2 LOW for RX
  delay(delayValue);
  digitalWrite(LED1, LOW);     // LED1 OFF
  digitalWrite(event1, HIGH);  // event1 HIGH for RX
}
